"""
Base class for module.
"""

from types import CodeType
from typing import List, Optional, Tuple

import importlib_metadata


class Module:
    """
    The Module class.
    """

    def __init__(
        self,
        name: str,
        path: Optional[str] = None,
        file_name: Optional[str] = None,
        parent: Optional["Module"] = None,
    ):
        self.name: str = name
        self.file: Optional[str] = file_name
        self.path: Optional[str] = path
        self.code: Optional[CodeType] = None
        self.parent: Optional["Module"] = parent
        self.global_names: set = set()
        self.exclude_names: set = set()
        self.ignore_names: set = set()
        self.source_is_zip_file: bool = False
        self.in_import: bool = True
        self.store_in_file_system: bool = True
        # distribution files (metadata)
        dist_files = []
        packages = [name]
        try:
            requires = importlib_metadata.requires(packages[0])
        except importlib_metadata.PackageNotFoundError:
            requires = None
        if requires is not None:
            packages += [req.partition(" ")[0] for req in requires]
        for package_name in packages:
            try:
                files = importlib_metadata.files(package_name)
            except importlib_metadata.PackageNotFoundError:
                files = None
            if files is not None:
                # cache file names to use in write modules
                for file in files:
                    if not file.match("*.dist-info/*"):
                        continue
                    dist_path = str(file.locate())
                    arc_path = file.as_posix()
                    dist_files.append((dist_path, arc_path))
        self.dist_files: List[Tuple[str, str]] = dist_files

    def __repr__(self) -> str:
        parts = [f"name={self.name!r}"]
        if self.file is not None:
            parts.append(f"file={self.file!r}")
        if self.path is not None:
            parts.append(f"path={self.path!r}")
        return "<Module {}>".format(", ".join(parts))

    def AddGlobalName(self, name: str) -> None:
        self.global_names.add(name)

    def ExcludeName(self, name: str) -> None:
        self.exclude_names.add(name)

    def IgnoreName(self, name: str) -> None:
        self.ignore_names.add(name)

    @property
    def in_file_system(self) -> bool:
        if self.parent is not None:
            return self.parent.in_file_system
        if self.path is None or self.file is None:
            return False
        return self.store_in_file_system



#!/usr/bin/env python

from datetime import datetime
import sys

print("Hello from cx_Freeze")
print(
    "The current date is %s\n"
    % datetime.today().strftime("%B %d, %Y %H:%M:%S")
)

print(f"Executable: {sys.executable!r}\n")

import BUILD_CONSTANTS

excludedVars = [
    "__builtins__",
    "__cached__",
    "__doc__",
    "__loader__",
    "__package__",
    "__spec__",
]

print("== variables in BUILD_CONSTANTS ==\n")
for var in dir(BUILD_CONSTANTS):
    if var in excludedVars:
        continue
    attr = BUILD_CONSTANTS.__getattribute__(var)
    print(f"{var} = {attr!r}")



from datetime import datetime
import sys

print("Hello from cx_Freeze")
print(f"The current date is {datetime.today():%B %d, %Y %H:%M:%S}\n")

print(f"Executable: {sys.executable}")
print(f"Prefix: {sys.prefix}")
print(f"Default encoding: {sys.getdefaultencoding()}")
print(f"File system encoding: {sys.getfilesystemencoding()}\n")

print("ARGUMENTS:")
for arg in sys.argv:
    print(f"{arg}")
print()

print("PATH:")
for path in sys.path:
    print(f"{path}")
print()



"""
Initialization script for cx_Freeze which manipulates the path so that the
directory in which the executable is found is searched for extensions but
no other directory is searched. The environment variable LD_LIBRARY_PATH is
manipulated first, however, to ensure that shared libraries found in the
target directory are found. This requires a restart of the executable because
the environment variable LD_LIBRARY_PATH is only checked at startup.

"""

import os
import sys

import BUILD_CONSTANTS

DIR_NAME = os.path.dirname(sys.executable)

paths = os.environ.get("LD_LIBRARY_PATH", "").split(os.pathsep)

if DIR_NAME not in paths:
    paths.insert(0, DIR_NAME)
    os.environ["LD_LIBRARY_PATH"] = os.pathsep.join(paths)
    os.execv(sys.executable, sys.argv)

sys.frozen = True
sys.path = sys.path[:4]

if hasattr(BUILD_CONSTANTS, "TCL_LIBRARY"):
    os.environ["TCL_LIBRARY"] = os.path.join(
        DIR_NAME, BUILD_CONSTANTS.TCL_LIBRARY
    )

if hasattr(BUILD_CONSTANTS, "TK_LIBRARY"):
    os.environ["TK_LIBRARY"] = os.path.join(
        DIR_NAME, BUILD_CONSTANTS.TK_LIBRARY
    )

if hasattr(BUILD_CONSTANTS, "PYTZ_TZDATADIR"):
    os.environ["PYTZ_TZDATADIR"] = os.path.join(
        DIR_NAME, BUILD_CONSTANTS.PYTZ_TZDATADIR
    )


def run():
    name = __name__.rpartition("__init__")[0] + "__main__"
    code = __loader__.get_code(name)
    m = __import__("__main__")
    m.__dict__["__file__"] = code.co_filename
    exec(code, m.__dict__)



"""A simple script to demonstrate matplotlib."""

from numpy import arange, sin, pi
import matplotlib

matplotlib.use("WXAgg")
from matplotlib.backends.backend_wxagg import FigureCanvasWxAgg as FigureCanvas
from matplotlib.backends.backend_wx import NavigationToolbar2Wx
from matplotlib.figure import Figure
import wx


class CanvasFrame(wx.Frame):
    def __init__(self):
        wx.Frame.__init__(self, None, -1, "CanvasFrame", size=(550, 350))
        color = wx.Colour("WHITE")
        self.SetBackgroundColour(color)
        self.figure = Figure()
        self.axes = self.figure.add_subplot(111)
        t = arange(0.0, 3.0, 0.01)
        s = sin(2 * pi * t)
        self.axes.plot(t, s)
        self.canvas = FigureCanvas(self, -1, self.figure)
        self.sizer = wx.BoxSizer(wx.VERTICAL)
        self.sizer.Add(self.canvas, 1, wx.LEFT | wx.TOP | wx.GROW)
        self.SetSizerAndFit(self.sizer)
        self.add_toolbar()

    def add_toolbar(self):
        self.toolbar = NavigationToolbar2Wx(self.canvas)
        self.toolbar.Realize()
        if wx.Platform == "__WXMAC__":
            self.SetToolBar(self.toolbar)
        else:
            tw, th = self.toolbar.GetSize()
            fw, fh = self.canvas.GetSize()
            self.toolbar.SetSize(wx.Size(fw, th))
            self.sizer.Add(self.toolbar, 0, wx.LEFT | wx.EXPAND)
        self.toolbar.update()

    def OnPaint(self, event):
        self.canvas.draw()


class App(wx.App):
    def OnInit(self):
        """Create the main window and insert the custom frame"""
        frame = CanvasFrame()
        frame.Show(True)
        return True


app = App(0)
app.MainLoop()



import os.path

from PIL import Image

print("Opening image with PIL")
filename = os.path.join("..", "icon", "favicon.png")
with Image.open(filename) as im, open("test_pillow.pdf", "w+b") as fp:
    if im.mode == "RGBA":
        im = im.convert("RGB")
    im.save(fp, format="PDF")
print("OK")



"""A simple setup script to create two executables using matplotlib."""

# matplotlib_eg.py is a very simple matplotlib application that demonstrates
# its use.
#
# matplotlib_afm.py is a very simple matplotlib console application that show
# some values of a font.
#
# Run the build process by running the command 'python setup.py build'
#
# If everything works well you should find a subdirectory in the build
# subdirectory that contains the files needed to run the application

import sys
from cx_Freeze import setup, Executable

base = "Console"
if sys.platform == "win32":
    base = "Win32GUI"

options = {
    "build_exe": {
        # Sometimes a little fine-tuning is needed
        # exclude all backends except wx
        "excludes": ["gtk", "PyQt4", "PyQt5", "tkinter"]
    }
}

executables = [
    Executable("matplotlib_afm.py"),
    Executable("matplotlib_eg.py", base=base),
]

setup(
    name="matplotlib_samples",
    version="0.1",
    description="Sample matplotlib script",
    executables=executables,
    options=options,
)



"""
A setup script to create two executables and demonstrate the use a valid and
an invalid icon.
"""

from cx_Freeze import setup, Executable

executables = [
    Executable("test_icon.py", icon="icon.ico", target_name="test_1.exe"),
    Executable("test_icon.py", icon="favicon.png", target_name="test_2.exe"),
]

setup(
    name="Icon sample",
    version="0.1",
    description="Test Icon",
    executables=executables,
)



# Code to test that the plist_items option is working correctly.

import setuptools
import distutils.core
import os, platform, sys
from data import TEST_KEY, TEST_VALUE, BUILD_DIR, BUNDLE_NAME

if platform.system() != "Darwin":
    print("Test only applies to Darwin.")
    sys.exit(1)

import plistlib

print("Testing plist_items option with bdist_mac.")

distutils.core.run_setup("setup.py", ["bdist_mac"])

# Test that the additional keys were correctly added to the plist.
name = os.path.join(BUILD_DIR, f"{BUNDLE_NAME}.app", "Contents", "Info.plist")
with open(name, "rb") as f:
    contents = plistlib.load(f, fmt=None, use_builtin_types=False)

try:
    assert contents[TEST_KEY] == TEST_VALUE
    print("Test Successful")
except AssertionError:
    print("Error, keys not correctly added to plist.")



import os
import subprocess
import stat
from typing import List, Tuple, Dict, Optional, Set, Iterable


# In a MachO file, need to deal specially with links that use @executable_path,
# @loader_path, @rpath
#
# @executable_path - where ultimate calling executable is
# @loader_path - directory of current object
# @rpath - list of paths to check
# (earlier rpaths have higher priority, i believe)
#
# Resolving these variables (particularly @rpath) requires tracing through the
# sequence linked MachO files leading the the current file, to determine which
# directories are included in the current rpath.


class DarwinException(Exception):
    pass


def _isMachOFile(path: str) -> bool:
    """Determines whether the file is a Mach-O file."""
    if not os.path.isfile(path):
        return False
    p = subprocess.Popen(("file", path), stdout=subprocess.PIPE)
    if b"Mach-O" in p.stdout.readline():
        return True
    return False


class MachOReference:
    """Represents a linking reference from MachO file to another file."""

    def __init__(
        self, sourceFile: "DarwinFile", rawPath: str, resolvedPath: str
    ):
        """
        :param sourceFile: DarwinFile object for file in which the reference
        was found
        :param rawPath: The path that appears in the file
        (may include @rpath, etc.)
        :param resolvedPath: The path resolved to an explicit path to a file
        on system.
        """
        self.sourceFile: "DarwinFile" = sourceFile
        self.rawPath: str = rawPath
        self.resolvedPath: str = resolvedPath

        # isSystemFile is True if the target is a system file that will not be
        # included in package
        self.isSystemFile = False
        # True if the file is being copied into the package
        self.isCopied = False
        # reference to target DarwinFile (but only if file is copied into app)
        self.targetFile: Optional[DarwinFile] = None

    def setTargetFile(self, darwinFile: "DarwinFile"):
        self.targetFile = darwinFile
        self.isCopied = True


class DarwinFile:
    """
    A DarwinFile object tracks a file referenced in the application, and record
    where it was ultimately moved to in the application bundle. Should also
    save a copy of the DarwinFile object, if any!, created for each referenced
    library.
    """

    def __init__(
        self,
        originalFilePath: str,
        referencingFile: Optional["DarwinFile"] = None,
    ):
        """
        :param originalFilePath: The original path of the DarwinFile (before
        copying into app)
        :param referencingFile: DarwinFile object representing the referencing
        source file
        """
        self.originalFilePath = os.path.realpath(originalFilePath)
        # path to file in build directory
        self._buildPath: Optional[str] = None
        self.commands: List[MachOCommand] = []
        self.loadCommands: List[MachOLoadCommand] = []
        self.rpathCommands: List[MachORPathCommand] = []

        # note: if file gets referenced twice (or more), it will only be the
        # first reference that gets recorded.
        self.referencingFile: Optional[DarwinFile] = None
        self.libraryPathResolution: Dict[str, str] = {}
        self._rpath: Optional[List[str]] = None

        # dictionary of MachOReference objects, by their resolved paths
        self.machOReferenceDict: Dict[str, MachOReference] = {}
        self.isMachO = False

        if not _isMachOFile(path=self.originalFilePath):
            return

        # if this is a MachO file, extract linking information from it
        self.isMachO = True
        self.commands = MachOCommand._getMachOCommands(
            forFileAtPath=self.originalFilePath
        )
        self.loadCommands = [
            c for c in self.commands if isinstance(c, MachOLoadCommand)
        ]
        self.rpathCommands = [
            c for c in self.commands if isinstance(c, MachORPathCommand)
        ]
        self.referencingFile = referencingFile

        self.getRPath()
        self.resolveLibraryPaths()

        for rawPath, resolvedPath in self.libraryPathResolution.items():
            if resolvedPath in self.machOReferenceDict:
                raise DarwinException(
                    "Dynamic libraries resolved to the same file?"
                )
            self.machOReferenceDict[resolvedPath] = MachOReference(
                sourceFile=self, rawPath=rawPath, resolvedPath=resolvedPath
            )
        return

    def __str__(self):
        l = []
        # l.append("RPath Commands: {}".format(self.rpathCommands))
        # l.append("Load commands: {}".format(self.loadCommands))
        l.append(f"Mach-O File: {self.originalFilePath}")
        l.append("Resolved rpath:")
        for rp in self.getRPath():
            l.append(f"   {rp}")
        l.append("Loaded libraries:")
        for rp in self.libraryPathResolution:
            l.append("   {} -> {}".format(rp, self.libraryPathResolution[rp]))
        return "\n".join(l)

    def setBuildPath(self, path: str):
        self._buildPath = path

    def getBuildPath(self) -> Optional[str]:
        return self._buildPath

    @staticmethod
    def isExecutablePath(path: str) -> bool:
        return path.startswith("@executable_path")

    @staticmethod
    def isLoaderPath(path: str) -> bool:
        return path.startswith("@loader_path")

    @staticmethod
    def isRPath(path: str) -> bool:
        return path.startswith("@rpath")

    def sourceDir(self) -> str:
        return os.path.dirname(self.originalFilePath)

    def resolveLoader(self, path: str) -> Optional[str]:
        """Resolve a path that includes @loader_path.
        @loader_path represents the directory in which the DarwinFile is
        located."""
        if self.isLoaderPath(path=path):
            return path.replace("@loader_path", self.sourceDir(), 1)
        raise DarwinException(f"resolveLoader() called on bad path: {path}")

    def resolveExecutable(self, path: str) -> str:
        """
        @executable_path should resolve to the directory where the original
        executable was located. By default, we set that to the directory of the
        library, so it would resolve in the same was as if linked from an
        executable in the same directory.
        """
        # consider making this resolve to the directory of the target script
        # instead?
        if self.isExecutablePath(path=path):
            return path.replace("@executable_path", self.sourceDir(), 1)
        raise DarwinException(
            f"resolveExecutable() called on bad path: {path}"
        )

    def resolveRPath(self, path: str) -> str:
        for rp in self.getRPath():
            testPath = os.path.abspath(path.replace("@rpath", rp, 1))
            if _isMachOFile(testPath):
                return testPath
        raise DarwinException(f"resolveRPath() failed to resolve path: {path}")

    def getRPath(self) -> List[str]:
        """
        Returns the rpath in effect for this file.  Determined by rpath
        commands in this file and (recursively) the chain of files that
        referenced this file.
        """
        if self._rpath is not None:
            return self._rpath
        rawPaths = [c.rPath for c in self.rpathCommands]
        rpath = []
        for rp in rawPaths:
            if os.path.isabs(rp):
                rpath.append(rp)
            elif self.isLoaderPath(rp):
                rpath.append(self.resolveLoader(rp))
            elif self.isExecutablePath(rp):
                rpath.append(self.resolveExecutable(rp))

        rpath = [os.path.abspath(rp) for rp in rpath]
        rpath = [rp for rp in rpath if os.path.exists(rp)]

        if self.referencingFile is not None:
            rpath = self.referencingFile.getRPath() + rpath
        self._rpath = rpath
        return self._rpath

    def resolvePath(self, path) -> str:
        """
        Resolves any @executable_path, @loader_path, and @rpath references
        in a path.
        """
        if self.isLoaderPath(path):  # replace @loader_path
            return self.resolveLoader(path)
        if self.isExecutablePath(path):  # replace @executable_path
            return self.resolveExecutable(path)
        if self.isRPath(path):  # replace @rpath
            return self.resolveRPath(path)
        if os.path.isabs(path):  # just use the path, if it is absolute
            return path
        testPath = os.path.abspath(os.path.join(self.sourceDir(), path))
        if _isMachOFile(path=testPath):
            return testPath
        raise DarwinException(f"Could not resolve path: {path}")

    def resolveLibraryPaths(self):
        for lc in self.loadCommands:
            rawPath = lc.loadPath
            resolvedPath = self.resolvePath(path=rawPath)
            self.libraryPathResolution[rawPath] = resolvedPath

    def getDependentFilePaths(self) -> List[str]:
        dependents: List[str] = []
        for ref in self.machOReferenceDict.values():
            dependents.append(ref.resolvedPath)
        return dependents

    def getMachOReference(self, resolvedPath: str) -> MachOReference:
        if resolvedPath not in self.machOReferenceDict:
            raise DarwinException(
                f"Path {resolvedPath} is not a path referenced from DarwinFile"
            )
        return self.machOReferenceDict[resolvedPath]

    def getMachOReferences(self) -> Iterable[Tuple[str, MachOReference]]:
        return self.machOReferenceDict.items()

    def setCopyDestination(self, destinationPath: str):
        """
        Tell the Mach-O file its relative position (compared to executable)
        in the bundled package.
        """
        self._buildPath = destinationPath


class MachOCommand:
    """Represents a load command in a MachO file."""

    def __init__(self, lines: List[str]):
        self.lines = lines

    def __repr__(self):
        return "<MachOCommand>"

    @staticmethod
    def _getMachOCommands(forFileAtPath: str) -> List["MachOCommand"]:
        """
        Returns a list of load commands in the specified file, using otool.
        """
        shellCommand = f'otool -l "{forFileAtPath}"'
        commands: List[MachOCommand] = []
        currentCommandLines = None

        # split the output into separate load commands
        for line in os.popen(shellCommand):
            line = line.strip()
            if line[:12] == "Load command":
                if currentCommandLines is not None:
                    commands.append(
                        MachOCommand.parseLines(lines=currentCommandLines)
                    )
                currentCommandLines = []
            if currentCommandLines is not None:
                currentCommandLines.append(line)
        if currentCommandLines is not None:
            commands.append(currentCommandLines)
        return commands

    @staticmethod
    def parseLines(lines: List[str]) -> "MachOCommand":
        if len(lines) < 2:
            return MachOCommand(lines=lines)
        commandLinePieces = lines[1].split(" ")
        if commandLinePieces[0] != "cmd":
            return MachOCommand(lines=lines)
        if commandLinePieces[1] == "LC_LOAD_DYLIB":
            return MachOLoadCommand(lines=lines)
        if commandLinePieces[1] == "LC_RPATH":
            return MachORPathCommand(lines=lines)
        return MachOCommand(lines=lines)


class MachOLoadCommand(MachOCommand):
    def __init__(self, lines: List[str]):
        super().__init__(lines=lines)
        self.loadPath = None
        if len(self.lines) < 4:
            return
        pathline = self.lines[3]
        pathline = pathline.strip()
        if not pathline.startswith("name "):
            return
        pathline = pathline[4:].strip()
        pathline = pathline.split("(offset")[0].strip()
        self.loadPath = pathline
        return

    def getPath(self):
        return self.loadPath

    def __repr__(self):
        return f"<LoadCommand path={self.loadPath!r}>"


class MachORPathCommand(MachOCommand):
    def __init__(self, lines: List[str]):
        super().__init__(lines=lines)
        self.rPath = None
        if len(self.lines) < 4:
            return
        pathline = self.lines[3]
        pathline = pathline.strip()
        if not pathline.startswith("path "):
            return
        pathline = pathline[4:].strip()
        pathline = pathline.split("(offset")[0].strip()
        self.rPath = pathline
        return

    def __repr__(self):
        return f"<RPath path={self.rPath!r}>"


def _printFile(
    darwinFile: DarwinFile,
    seenFiles: Set[DarwinFile],
    level: int,
    noRecurse=False,
):
    """
    Utility function to prints details about a DarwinFile and (optionally)
    recursively any other DarwinFiles that it references.
    """
    print("{}{}".format(level * "|  ", darwinFile.originalFilePath), end="")
    print(" (already seen)" if noRecurse else "")
    if noRecurse:
        return
    for ref in darwinFile.machOReferenceDict.values():
        if not ref.isCopied:
            continue
        mf = ref.targetFile
        _printFile(
            mf,
            seenFiles=seenFiles,
            level=level + 1,
            noRecurse=(mf in seenFiles),
        )
        seenFiles.add(mf)
    return


def printMachOFiles(fileList: List[DarwinFile]):
    seenFiles = set()
    for mf in fileList:
        if mf not in seenFiles:
            seenFiles.add(mf)
            _printFile(mf, seenFiles=seenFiles, level=0)


def changeLoadReference(
    fileName: str, oldReference: str, newReference: str, VERBOSE: bool = True
):
    """
    Utility function that uses intall_name_tool to change oldReference to
    newReference in the machO file specified by fileName.
    """
    if VERBOSE:
        print("Redirecting load reference for ", end="")
        print(f"<{fileName}> {oldReference} -> {newReference}")
    original = os.stat(fileName).st_mode
    newMode = original | stat.S_IWUSR
    os.chmod(fileName, newMode)
    subprocess.call(
        ("install_name_tool", "-change", oldReference, newReference, fileName)
    )
    os.chmod(fileName, original)


class DarwinFileTracker:
    """Object to track the DarwinFiles that have been added during a freeze."""

    def __init__(self):
        # a list of DarwinFile objects for files being copied into project
        self._targetFileList: List[DarwinFile] = []

        # a mapping of (build directory) target paths to DarwinFile objects
        self._targetFileDict: Dict[str, DarwinFile] = {}

        self._sourceFileDict: Dict[str, DarwinFile] = {}

        # a mapping of (source location) paths to the MacOReferences to them
        self._referenceCache: Dict[str, MachOReference] = {}

    def __iter__(self) -> Iterable[DarwinFile]:
        return iter(self._targetFileList)

    def pathIsAlreadyCopiedTo(self, targetPath: str) -> bool:
        """Check if the given targetPath has already has a file copied to it."""
        if targetPath in self._targetFileDict:
            return True
        return False

    def getDarwinFile(self, sourcePath: str, targetPath: str) -> DarwinFile:
        """
        Gets the DarwinFile for file copied from sourcePath to targetPath.
        If either (i) nothing, or (ii) a different file has been copied to
        targetPath, raises a DarwinException.
        """

        # check that the file has been copied to
        if targetPath not in self._targetFileDict:
            raise DarwinException(
                f"File {targetPath!r} already copied to, "
                "but no DarwinFile object found for it."
            )

        # check that the target file came from the specified source
        targetDarwinFile: DarwinFile = self._targetFileDict[targetPath]
        realSource = os.path.realpath(sourcePath)
        targetRealSource = os.path.realpath(targetDarwinFile.originalFilePath)
        if realSource != targetRealSource:
            raise DarwinException(
                f"Attempting to copy two files to {targetPath!r}\n"
                f"source 1: {targetDarwinFile.originalFilePath!r} "
                f"(real: {targetRealSource!r})\n"
                f"source 2: {sourcePath!r} (real: {realSource!r})\n"
                "(This may be caused by including modules in the zip file "
                "that rely on binary libraries with the same name.)"
            )
        return targetDarwinFile

    def recordCopiedFile(self, targetPath: str, darwinFile: DarwinFile):
        """
        Record that a DarwinFile is being copied to a given path.  If the
        same file has already been copied to that path, do nothing. If a
        different file has been copied to that bath, raise a DarwinException.
        """
        if self.pathIsAlreadyCopiedTo(targetPath=targetPath):
            raise DarwinException(
                "addFile() called with targetPath already copied to "
                f"(targetPath={targetPath!r})"
            )

        self._targetFileList.append(darwinFile)
        self._targetFileDict[targetPath] = darwinFile
        self._sourceFileDict[darwinFile.originalFilePath] = darwinFile

    def cacheReferenceTo(self, path: str, machOReference: MachOReference):
        self._referenceCache[path] = machOReference

    def getCachedReferenceTo(self, path: str) -> Optional[MachOReference]:
        if path in self._referenceCache:
            return self._referenceCache[path]
        return None

    def finalizeReferences(self):
        """
        Goes through the stored list of target files, and updates references.
        Normally the references may not be updated automatically, if _CopyFile
        is called without copyDependentFiles=True.
        """

        for df in self._targetFileList:  # DarwinFile
            for (path, ref) in df.getMachOReferences():
                # path and corresponding MachOReference to path
                if not ref.isCopied:
                    # if reference not already marked as copied, check if it
                    # points to a file that is being copied and, if so, use
                    # that to complete the reference
                    realTargetPath = os.path.realpath(path)
                    if realTargetPath in self._sourceFileDict:
                        ref.setTargetFile(self._sourceFileDict[realTargetPath])



# A simple script demonstrating use of plist_items option.
#
# hello.py is a very simple 'Hello, world' type script which also displays the
# environment in which the script runs
#
# Run the build process by running the command 'python setup.py build'
#
# If everything works well you should find a subdirectory in the build
# subdirectory that contains the files needed to run the script without Python

from cx_Freeze import setup, Executable
from data import TEST_KEY, TEST_VALUE, BUILD_DIR, BUNDLE_NAME

executables = [Executable("hello.py")]

setup(
    name="hello",
    version="0.1",
    description="Sample cx_Freeze script",
    options={
        "build": {"build_base": BUILD_DIR},
        "bdist_mac": {
            "bundle_name": BUNDLE_NAME,
            "plist_items": [(TEST_KEY, TEST_VALUE)],
        },
    },
    executables=executables,
)



#!/usr/bin/env python

import sys
from PySide2.QtWidgets import QApplication, QWidget

# import QtGui and QtCore only so that they are also frozen
import PySide2.QtGui
import PySide2.QtCore


def main():
    app = QApplication(sys.argv)
    window = QWidget()
    window.setWindowTitle("Simple")
    window.show()
    return app.exec_()


if __name__ == "__main__":
    sys.exit(main())



import sys, os, site
from cx_Freeze import setup, Executable
from typing import List, Tuple

PACKAGENAME = "TEST"
INCLUDE_QT = True  # whether to include certain PyQT-related files


def getQtPluginIncludes(pluginList: List[str]) -> List[Tuple[str, str]]:
    includes = []
    for ppath in pluginList:
        includes.append(_getInclude(ppath))
        pass
    return includes


def _getInclude(pluginPath: str) -> Tuple[str, str]:
    foundPath = None

    if sys.platform == "darwin":
        packagesDirs = [c for c in sys.path if c.find("site-packages") != -1]
    else:
        # search site packages locations to see if we can find required .dll
        packagesDirs = site.getsitepackages()
        pass
    for pdir in packagesDirs:
        testPath = os.path.join(
            pdir, os.path.join("PyQt5", "Qt", "plugins", pluginPath)
        )

        # print("Checking for {} at {}".format(bname, testPath))
        if os.path.exists(testPath):
            foundPath = testPath
            # print("DLL Found")
            break
        pass
    if foundPath is None:
        print(f"Error, could not find: {pluginPath}")
        sys.exit(1)

    return (foundPath, pluginPath)


# force the inclusion of certain plugins that cx-freeze cannot find on its own
requiredPlugins = [
    "styles/libqmacstyle.dylib",
    "sqldrivers/libqsqlite.dylib",
    "printsupport/libcocoaprintersupport.dylib",
]

if not INCLUDE_QT:
    requiredPlugins = []
include_files = []

include_files += getQtPluginIncludes(pluginList=requiredPlugins)

keyFiles = []

build_options = {
    "build_exe": "cx_build/",  # subdirectory to do build in
    "build_base": "cx_build_dists",  # subdirectory to place .app and .dmg packages in
}
# Cause the PyQt5 to be included in the package in the old way (only specifically required files).
# This makes the package *much* smaller.
zip_include_packages = ["PyQt5"]
extraPackages = ["PyQt5.sip"]  # force PyQt5.sip to be included.

if not INCLUDE_QT:
    zip_include_packages = []
    extraPackages = []


build_exe_options = {
    "include_files": include_files,
    "zip_include_packages": zip_include_packages,
    "packages": extraPackages,
}

bdist_mac_options = {
    "bundle_name": "Test",
}

bdist_dmg_options = {
    "volume_label": PACKAGENAME,
}

exe = Executable(script="test_script.py")
exe2 = Executable(script="test_script2.py")


setup(
    name="Test application",
    author="[author]",
    maintainer="[maintainer]",
    maintainer_email="[email]",
    options={
        "build": build_options,
        "build_exe": build_exe_options,
        "bdist_mac": bdist_mac_options,
        "bdist_dmg": bdist_dmg_options,
    },
    executables=[exe, exe2],
)



"""
A setup script to demonstrate build using pytz
This version requires the zoneinfo in the file system
"""
#
# Run the build process by running the command 'python setup.py build'
#
# If everything works well you should find a subdirectory in the build
# subdirectory that contains the files needed to run the script without Python

from cx_Freeze import setup, Executable

setup(
    name="test_pytz_zip",
    version="0.3",
    description="cx_Freeze script to test pytz",
    executables=[Executable("test_pytz.py")],
)



import distutils.command.bdist_msi
import distutils.errors
import distutils.util
import importlib
import msilib
import os
import re

__all__ = ["bdist_msi"]

# force the remove existing products action to happen first since Windows
# installer appears to be braindead and doesn't handle files shared between
# different "products" very well
sequence = msilib.sequence.InstallExecuteSequence
for index, info in enumerate(sequence):
    if info[0] == "RemoveExistingProducts":
        sequence[index] = (info[0], info[1], 1450)


class bdist_msi(distutils.command.bdist_msi.bdist_msi):
    user_options = distutils.command.bdist_msi.bdist_msi.user_options + [
        ("add-to-path=", None, "add target dir to PATH environment variable"),
        ("upgrade-code=", None, "upgrade code to use"),
        ("initial-target-dir=", None, "initial target directory"),
        ("target-name=", None, "name of the file to create"),
        ("directories=", None, "list of 3-tuples of directories to create"),
        ("environment-variables=", None, "list of environment variables"),
        (
            "data=",
            None,
            "dictionary of data indexed by table name, and each value is a "
            "tuple to include in table",
        ),
        (
            "summary-data=",
            None,
            "Dictionary of data to include in msi summary data stream. "
            'Allowed keys are "author", "comments", "keywords".',
        ),
        ("product-code=", None, "product code to use"),
        ("install-icon=", None, "icon path to add/remove programs "),
        ("all-users=", None, "installation for all users (or just me)"),
    ]
    x = y = 50
    width = 370
    height = 300
    title = "[ProductName] Setup"
    modeless = 1
    modal = 3

    def add_config(self, fullname):
        if self.add_to_path:
            p = "Path"
            if self.all_users:
                p = "=-*" + p
            msilib.add_data(
                self.db,
                "Environment",
                [("E_PATH", p, r"[~];[TARGETDIR]", "TARGETDIR")],
            )
        if self.directories:
            msilib.add_data(self.db, "Directory", self.directories)
        if self.environment_variables:
            msilib.add_data(self.db, "Environment", self.environment_variables)
        # This is needed in case the AlwaysInstallElevated policy is set.
        # Otherwise installation will not end up in TARGETDIR.
        msilib.add_data(
            self.db, "Property", [("SecureCustomProperties", "TARGETDIR")]
        )
        msilib.add_data(
            self.db,
            "CustomAction",
            [
                (
                    "A_SET_TARGET_DIR",
                    256 + 51,
                    "TARGETDIR",
                    self.initial_target_dir,
                )
            ],
        )
        msilib.add_data(
            self.db,
            "InstallExecuteSequence",
            [("A_SET_TARGET_DIR", 'TARGETDIR=""', 401)],
        )
        msilib.add_data(
            self.db,
            "InstallUISequence",
            [
                ("PrepareDlg", None, 140),
                ("A_SET_TARGET_DIR", 'TARGETDIR=""', 401),
                ("SelectDirectoryDlg", "not Installed", 1230),
                (
                    "MaintenanceTypeDlg",
                    "Installed and not Resume and not Preselected",
                    1250,
                ),
                ("ProgressDlg", None, 1280),
            ],
        )
        for index, executable in enumerate(self.distribution.executables):
            if (
                executable.shortcut_name is not None
                and executable.shortcut_dir is not None
            ):
                baseName = os.path.basename(executable.target_name)
                msilib.add_data(
                    self.db,
                    "Shortcut",
                    [
                        (
                            "S_APP_%s" % index,
                            executable.shortcut_dir,
                            executable.shortcut_name,
                            "TARGETDIR",
                            "[TARGETDIR]%s" % baseName,
                            None,
                            None,
                            None,
                            None,
                            None,
                            None,
                            None,
                        )
                    ],
                )
        for tableName, data in self.data.items():
            msilib.add_data(self.db, tableName, data)

        # If provided, add data to MSI's summary information stream
        if len(self.summary_data) > 0:
            for k in self.summary_data:
                if k not in ["author", "comments", "keywords"]:
                    raise Exception(
                        f"Unknown key provided in summary-data: {k!r}"
                    )

            summaryInfo = self.db.GetSummaryInformation(5)
            if "author" in self.summary_data:
                summaryInfo.SetProperty(
                    msilib.PID_AUTHOR, self.summary_data["author"]
                )
            if "comments" in self.summary_data:
                summaryInfo.SetProperty(
                    msilib.PID_COMMENTS, self.summary_data["comments"]
                )
            if "keywords" in self.summary_data:
                summaryInfo.SetProperty(
                    msilib.PID_KEYWORDS, self.summary_data["keywords"]
                )
            summaryInfo.Persist()

    def add_cancel_dialog(self):
        dialog = msilib.Dialog(
            self.db,
            "CancelDlg",
            50,
            10,
            260,
            85,
            3,
            self.title,
            "No",
            "No",
            "No",
        )
        dialog.text(
            "Text",
            48,
            15,
            194,
            30,
            3,
            "Are you sure you want to cancel [ProductName] installation?",
        )
        button = dialog.pushbutton("Yes", 72, 57, 56, 17, 3, "Yes", "No")
        button.event("EndDialog", "Exit")
        button = dialog.pushbutton("No", 132, 57, 56, 17, 3, "No", "Yes")
        button.event("EndDialog", "Return")

    def add_error_dialog(self):
        dialog = msilib.Dialog(
            self.db,
            "ErrorDlg",
            50,
            10,
            330,
            101,
            65543,
            self.title,
            "ErrorText",
            None,
            None,
        )
        dialog.text("ErrorText", 50, 9, 280, 48, 3, "")
        for text, x in [
            ("No", 120),
            ("Yes", 240),
            ("Abort", 0),
            ("Cancel", 42),
            ("Ignore", 81),
            ("Ok", 159),
            ("Retry", 198),
        ]:
            button = dialog.pushbutton(text[0], x, 72, 81, 21, 3, text, None)
            button.event("EndDialog", "Error%s" % text)

    def add_exit_dialog(self):
        dialog = distutils.command.bdist_msi.PyDialog(
            self.db,
            "ExitDialog",
            self.x,
            self.y,
            self.width,
            self.height,
            self.modal,
            self.title,
            "Finish",
            "Finish",
            "Finish",
        )
        dialog.title("Completing the [ProductName] installer")
        dialog.back("< Back", "Finish", active=False)
        dialog.cancel("Cancel", "Back", active=False)
        dialog.text(
            "Description",
            15,
            235,
            320,
            20,
            0x30003,
            "Click the Finish button to exit the installer.",
        )
        button = dialog.next("Finish", "Cancel", name="Finish")
        button.event("EndDialog", "Return")

    def add_fatal_error_dialog(self):
        dialog = distutils.command.bdist_msi.PyDialog(
            self.db,
            "FatalError",
            self.x,
            self.y,
            self.width,
            self.height,
            self.modal,
            self.title,
            "Finish",
            "Finish",
            "Finish",
        )
        dialog.title("[ProductName] installer ended prematurely")
        dialog.back("< Back", "Finish", active=False)
        dialog.cancel("Cancel", "Back", active=False)
        dialog.text(
            "Description1",
            15,
            70,
            320,
            80,
            0x30003,
            "[ProductName] setup ended prematurely because of an error. "
            "Your system has not been modified. To install this program "
            "at a later time, please run the installation again.",
        )
        dialog.text(
            "Description2",
            15,
            155,
            320,
            20,
            0x30003,
            "Click the Finish button to exit the installer.",
        )
        button = dialog.next("Finish", "Cancel", name="Finish")
        button.event("EndDialog", "Exit")

    def add_files(self):
        db = self.db
        cab = msilib.CAB("distfiles")
        f = msilib.Feature(
            db,
            "default",
            "Default Feature",
            "Everything",
            1,
            directory="TARGETDIR",
        )
        f.set_current()
        rootdir = os.path.abspath(self.bdist_dir)
        root = msilib.Directory(
            db, cab, None, rootdir, "TARGETDIR", "SourceDir"
        )
        db.Commit()
        todo = [root]
        while todo:
            dir = todo.pop()
            for file in os.listdir(dir.absolute):
                if os.path.isdir(os.path.join(dir.absolute, file)):
                    newDir = msilib.Directory(
                        db,
                        cab,
                        dir,
                        file,
                        file,
                        "{}|{}".format(dir.make_short(file), file),
                    )
                    todo.append(newDir)
                else:
                    dir.add_file(file)
        cab.commit(db)

    def add_files_in_use_dialog(self):
        dialog = distutils.command.bdist_msi.PyDialog(
            self.db,
            "FilesInUse",
            self.x,
            self.y,
            self.width,
            self.height,
            19,
            self.title,
            "Retry",
            "Retry",
            "Retry",
            bitmap=False,
        )
        dialog.text(
            "Title", 15, 6, 200, 15, 0x30003, r"{\DlgFontBold8}Files in Use"
        )
        dialog.text(
            "Description",
            20,
            23,
            280,
            20,
            0x30003,
            "Some files that need to be updated are currently in use.",
        )
        dialog.text(
            "Text",
            20,
            55,
            330,
            50,
            3,
            "The following applications are using files that need to be "
            "updated by this setup. Close these applications and then "
            "click Retry to continue the installation or Cancel to exit "
            "it.",
        )
        dialog.control(
            "List",
            "ListBox",
            20,
            107,
            330,
            130,
            7,
            "FileInUseProcess",
            None,
            None,
            None,
        )
        button = dialog.back("Exit", "Ignore", name="Exit")
        button.event("EndDialog", "Exit")
        button = dialog.next("Ignore", "Retry", name="Ignore")
        button.event("EndDialog", "Ignore")
        button = dialog.cancel("Retry", "Exit", name="Retry")
        button.event("EndDialog", "Retry")

    def add_maintenance_type_dialog(self):
        dialog = distutils.command.bdist_msi.PyDialog(
            self.db,
            "MaintenanceTypeDlg",
            self.x,
            self.y,
            self.width,
            self.height,
            self.modal,
            self.title,
            "Next",
            "Next",
            "Cancel",
        )
        dialog.title("Welcome to the [ProductName] Setup Wizard")
        dialog.text(
            "BodyText",
            15,
            63,
            330,
            42,
            3,
            "Select whether you want to repair or remove [ProductName].",
        )
        group = dialog.radiogroup(
            "RepairRadioGroup",
            15,
            108,
            330,
            60,
            3,
            "MaintenanceForm_Action",
            "",
            "Next",
        )
        group.add("Repair", 0, 18, 300, 17, "&Repair [ProductName]")
        group.add("Remove", 0, 36, 300, 17, "Re&move [ProductName]")
        dialog.back("< Back", None, active=False)
        button = dialog.next("Finish", "Cancel")
        button.event(
            "[REINSTALL]", "ALL", 'MaintenanceForm_Action="Repair"', 5
        )
        button.event(
            "[Progress1]", "Repairing", 'MaintenanceForm_Action="Repair"', 6
        )
        button.event(
            "[Progress2]", "repairs", 'MaintenanceForm_Action="Repair"', 7
        )
        button.event("Reinstall", "ALL", 'MaintenanceForm_Action="Repair"', 8)
        button.event("[REMOVE]", "ALL", 'MaintenanceForm_Action="Remove"', 11)
        button.event(
            "[Progress1]", "Removing", 'MaintenanceForm_Action="Remove"', 12
        )
        button.event(
            "[Progress2]", "removes", 'MaintenanceForm_Action="Remove"', 13
        )
        button.event("Remove", "ALL", 'MaintenanceForm_Action="Remove"', 14)
        button.event(
            "EndDialog", "Return", 'MaintenanceForm_Action<>"Change"', 20
        )
        button = dialog.cancel("Cancel", "RepairRadioGroup")
        button.event("SpawnDialog", "CancelDlg")

    def add_prepare_dialog(self):
        dialog = distutils.command.bdist_msi.PyDialog(
            self.db,
            "PrepareDlg",
            self.x,
            self.y,
            self.width,
            self.height,
            self.modeless,
            self.title,
            "Cancel",
            "Cancel",
            "Cancel",
        )
        dialog.text(
            "Description",
            15,
            70,
            320,
            40,
            0x30003,
            "Please wait while the installer prepares to guide you through "
            "the installation.",
        )
        dialog.title("Welcome to the [ProductName] installer")
        text = dialog.text(
            "ActionText", 15, 110, 320, 20, 0x30003, "Pondering..."
        )
        text.mapping("ActionText", "Text")
        text = dialog.text("ActionData", 15, 135, 320, 30, 0x30003, None)
        text.mapping("ActionData", "Text")
        dialog.back("Back", None, active=False)
        dialog.next("Next", None, active=False)
        button = dialog.cancel("Cancel", None)
        button.event("SpawnDialog", "CancelDlg")

    def add_progress_dialog(self):
        dialog = distutils.command.bdist_msi.PyDialog(
            self.db,
            "ProgressDlg",
            self.x,
            self.y,
            self.width,
            self.height,
            self.modeless,
            self.title,
            "Cancel",
            "Cancel",
            "Cancel",
            bitmap=False,
        )
        dialog.text(
            "Title",
            20,
            15,
            200,
            15,
            0x30003,
            r"{\DlgFontBold8}[Progress1] [ProductName]",
        )
        dialog.text(
            "Text",
            35,
            65,
            300,
            30,
            3,
            "Please wait while the installer [Progress2] [ProductName].",
        )
        dialog.text("StatusLabel", 35, 100, 35, 20, 3, "Status:")
        text = dialog.text(
            "ActionText", 70, 100, self.width - 70, 20, 3, "Pondering..."
        )
        text.mapping("ActionText", "Text")
        control = dialog.control(
            "ProgressBar",
            "ProgressBar",
            35,
            120,
            300,
            10,
            65537,
            None,
            "Progress done",
            None,
            None,
        )
        control.mapping("SetProgress", "Progress")
        dialog.back("< Back", "Next", active=False)
        dialog.next("Next >", "Cancel", active=False)
        button = dialog.cancel("Cancel", "Back")
        button.event("SpawnDialog", "CancelDlg")

    def add_properties(self):
        metadata = self.distribution.metadata
        props = [
            ("DistVersion", metadata.get_version()),
            ("DefaultUIFont", "DlgFont8"),
            ("ErrorDialog", "ErrorDlg"),
            ("Progress1", "Install"),
            ("Progress2", "installs"),
            ("MaintenanceForm_Action", "Repair"),
            ("ALLUSERS", "2"),
        ]

        if not self.all_users:
            props.append(("MSIINSTALLPERUSER", "1"))
        email = metadata.author_email or metadata.maintainer_email
        if email:
            props.append(("ARPCONTACT", email))
        if metadata.url:
            props.append(("ARPURLINFOABOUT", metadata.url))
        if self.upgrade_code is not None:
            if not is_valid_GUID(self.upgrade_code):
                raise ValueError("upgrade-code must be in valid GUID format")
            props.append(("UpgradeCode", self.upgrade_code.upper()))
        if self.install_icon:
            props.append(("ARPPRODUCTICON", "InstallIcon"))
        msilib.add_data(self.db, "Property", props)
        if self.install_icon:
            msilib.add_data(
                self.db,
                "Icon",
                [("InstallIcon", msilib.Binary(self.install_icon))],
            )

    def add_select_directory_dialog(self):
        dialog = distutils.command.bdist_msi.PyDialog(
            self.db,
            "SelectDirectoryDlg",
            self.x,
            self.y,
            self.width,
            self.height,
            self.modal,
            self.title,
            "Next",
            "Next",
            "Cancel",
        )
        dialog.title("Select destination directory")
        dialog.back("< Back", None, active=False)
        button = dialog.next("Next >", "Cancel")
        button.event("SetTargetPath", "TARGETDIR", ordering=1)
        button.event("SpawnWaitDialog", "WaitForCostingDlg", ordering=2)
        button.event("EndDialog", "Return", ordering=3)
        button = dialog.cancel("Cancel", "DirectoryCombo")
        button.event("SpawnDialog", "CancelDlg")
        dialog.control(
            "DirectoryCombo",
            "DirectoryCombo",
            15,
            70,
            272,
            80,
            393219,
            "TARGETDIR",
            None,
            "DirectoryList",
            None,
        )
        dialog.control(
            "DirectoryList",
            "DirectoryList",
            15,
            90,
            308,
            136,
            3,
            "TARGETDIR",
            None,
            "PathEdit",
            None,
        )
        dialog.control(
            "PathEdit",
            "PathEdit",
            15,
            230,
            306,
            16,
            3,
            "TARGETDIR",
            None,
            "Next",
            None,
        )
        button = dialog.pushbutton("Up", 306, 70, 18, 18, 3, "Up", None)
        button.event("DirectoryListUp", "0")
        button = dialog.pushbutton("NewDir", 324, 70, 30, 18, 3, "New", None)
        button.event("DirectoryListNew", "0")

    def add_text_styles(self):
        msilib.add_data(
            self.db,
            "TextStyle",
            [
                ("DlgFont8", "Tahoma", 9, None, 0),
                ("DlgFontBold8", "Tahoma", 8, None, 1),
                ("VerdanaBold10", "Verdana", 10, None, 1),
                ("VerdanaRed9", "Verdana", 9, 255, 0),
            ],
        )

    def add_ui(self):
        self.add_text_styles()
        self.add_error_dialog()
        self.add_fatal_error_dialog()
        self.add_cancel_dialog()
        self.add_exit_dialog()
        self.add_user_exit_dialog()
        self.add_files_in_use_dialog()
        self.add_wait_for_costing_dialog()
        self.add_prepare_dialog()
        self.add_select_directory_dialog()
        self.add_progress_dialog()
        self.add_maintenance_type_dialog()

    def add_upgrade_config(self, sversion):
        if self.upgrade_code is not None:
            msilib.add_data(
                self.db,
                "Upgrade",
                [
                    (
                        self.upgrade_code,
                        None,
                        sversion,
                        None,
                        513,
                        None,
                        "REMOVEOLDVERSION",
                    ),
                    (
                        self.upgrade_code,
                        sversion,
                        None,
                        None,
                        257,
                        None,
                        "REMOVENEWVERSION",
                    ),
                ],
            )

    def add_user_exit_dialog(self):
        dialog = distutils.command.bdist_msi.PyDialog(
            self.db,
            "UserExit",
            self.x,
            self.y,
            self.width,
            self.height,
            self.modal,
            self.title,
            "Finish",
            "Finish",
            "Finish",
        )
        dialog.title("[ProductName] installer was interrupted")
        dialog.back("< Back", "Finish", active=False)
        dialog.cancel("Cancel", "Back", active=False)
        dialog.text(
            "Description1",
            15,
            70,
            320,
            80,
            0x30003,
            "[ProductName] setup was interrupted. Your system has not "
            "been modified. To install this program at a later time, "
            "please run the installation again.",
        )
        dialog.text(
            "Description2",
            15,
            155,
            320,
            20,
            0x30003,
            "Click the Finish button to exit the installer.",
        )
        button = dialog.next("Finish", "Cancel", name="Finish")
        button.event("EndDialog", "Exit")

    def add_wait_for_costing_dialog(self):
        dialog = msilib.Dialog(
            self.db,
            "WaitForCostingDlg",
            50,
            10,
            260,
            85,
            self.modal,
            self.title,
            "Return",
            "Return",
            "Return",
        )
        dialog.text(
            "Text",
            48,
            15,
            194,
            30,
            3,
            "Please wait while the installer finishes determining your "
            "disk space requirements.",
        )
        button = dialog.pushbutton(
            "Return", 102, 57, 56, 17, 3, "Return", None
        )
        button.event("EndDialog", "Exit")

    def finalize_options(self):
        distutils.command.bdist_msi.bdist_msi.finalize_options(self)
        name = self.distribution.get_name()
        fullname = self.distribution.get_fullname()
        if self.initial_target_dir is None:
            if distutils.util.get_platform() == "win-amd64":
                programFilesFolder = "ProgramFiles64Folder"
            else:
                programFilesFolder = "ProgramFilesFolder"
            self.initial_target_dir = fr"[{programFilesFolder}]\{name}"
        if self.add_to_path is None:
            self.add_to_path = False
        if self.target_name is None:
            self.target_name = fullname
        if not self.target_name.lower().endswith(".msi"):
            platform = distutils.util.get_platform().replace("win-", "")
            self.target_name = f"{self.target_name}-{platform}.msi"
        if not os.path.isabs(self.target_name):
            self.target_name = os.path.join(self.dist_dir, self.target_name)
        if self.directories is None:
            self.directories = []
        if self.environment_variables is None:
            self.environment_variables = []
        if self.data is None:
            self.data = {}
        if not isinstance(self.summary_data, dict):
            self.summary_data = {}

    def initialize_options(self):
        distutils.command.bdist_msi.bdist_msi.initialize_options(self)
        self.upgrade_code = None
        self.product_code = None
        self.add_to_path = None
        self.initial_target_dir = None
        self.target_name = None
        self.directories = None
        self.environment_variables = None
        self.data = None
        self.summary_data = None
        self.install_icon = None
        self.all_users = False

    def run(self):
        if not self.skip_build:
            self.run_command("build")
        install = self.reinitialize_command("install", reinit_subcommands=1)
        install.prefix = self.bdist_dir
        install.skip_build = self.skip_build
        install.warn_dir = 0
        distutils.log.info("installing to %s", self.bdist_dir)
        install.ensure_finalized()
        install.run()
        self.mkpath(self.dist_dir)
        fullname = self.distribution.get_fullname()
        if os.path.exists(self.target_name):
            os.unlink(self.target_name)
        metadata = self.distribution.metadata
        author = metadata.author or metadata.maintainer or "UNKNOWN"
        version = metadata.get_version()
        sversion = ".".join(
            [str(x) for x in distutils.version.LooseVersion(version).version]
        )

        # msilib is reloaded in order to reset the "_directories" global member
        # in that module.  That member is used by msilib to prevent any two
        # directories from having the same logical name.  _directories might
        # already have contents due to msilib having been previously used in
        # the current instance of the python interpreter -- if so, it could
        # prevent the root from getting the logical name TARGETDIR, breaking
        # the MSI.
        importlib.reload(msilib)

        if self.product_code is None:
            self.product_code = msilib.gen_uuid()
        self.db = msilib.init_database(
            self.target_name,
            msilib.schema,
            self.distribution.metadata.name,
            self.product_code,
            sversion,
            author,
        )
        msilib.add_tables(self.db, msilib.sequence)
        self.add_properties()
        self.add_config(fullname)
        self.add_upgrade_config(sversion)
        self.add_ui()
        self.add_files()
        self.db.Commit()
        self.distribution.dist_files.append(
            ("bdist_msi", sversion or "any", self.target_name)
        )

        if not self.keep_temp:
            distutils.dir_util.remove_tree(
                self.bdist_dir, dry_run=self.dry_run
            )

        # Cause the MSI file to be released. Without this, then if bdist_msi
        # is run programmatically from within a larger script, subsequent
        # editting of the MSI is blocked.
        self.db = None


def is_valid_GUID(code):
    pattern = re.compile(
        r"^\{[0-9A-F]{8}-([0-9A-F]{4}-){3}[0-9A-F]{12}\}$", re.IGNORECASE
    )
    return isinstance(code, str) and pattern.match(code) is not None



"""
A setup script to demonstrate build using zoneinfo/tzdata
This version requires the zoneinfo in the file system
"""
#
# Run the build process by running the command 'python setup.py build'
#
# If everything works well you should find a subdirectory in the build
# subdirectory that contains the files needed to run the script without Python

from cx_Freeze import setup, Executable


setup(
    name="test_tz",
    version="0.1",
    description="cx_Freeze script to test zoneinfo/tzdata",
    executables=[Executable("test_tz.py")],
)



#!/usr/bin/env python

from datetime import datetime
import sys


def SayHello():
    print("Hello from cx_Freeze")
    print(
        "The current date is {}\n".format(
            datetime.today().strftime("%B %d, %Y %H:%M:%S")
        )
    )

    print(f"Executable: {sys.executable}")
    print(f"Prefix: {sys.prefix}")
    print(f"Default encoding: {sys.getdefaultencoding()}")
    print(f"File system encoding: {sys.getfilesystemencoding()}\n")

    print("ARGUMENTS:")
    for a in sys.argv:
        print(f"{a}")
    print("")

    print("PATH:")
    for p in sys.path:
        print(f"{p}")
    print()
    return


if __name__ == "__main__":
    SayHello()



"""
A setup script to demonstrate build using zoneinfo/tzdata
This version requires the zoneinfo in the zip file
"""
#
# Run the build process by running the command 'python setup.py build'
#
# If everything works well you should find a subdirectory in the build
# subdirectory that contains the files needed to run the script without Python

import distutils
import sys
import os

from cx_Freeze import setup, Executable

dir_name = "exe.{}-{}.zip".format(
    distutils.util.get_platform(), sys.version[0:3]
)
build_exe = os.path.join("build", dir_name)

setup(
    name="test_tz_zip",
    version="0.1",
    description="cx_Freeze script to test zoneinfo/tzdata",
    executables=[Executable("test_tz.py")],
    options={
        "build_exe": {
            "zip_include_packages": ["*"],
            "zip_exclude_packages": [],
            "build_exe": build_exe,
        }
    },
)



# A very simple setup script to test adding summary data stream to an MSI file.
#
# hello.py is a very simple 'Hello, world' type script which also displays the
# environment in which the script runs
#
# Run the build process by running the command 'python setup.py bdist_msi'


from cx_Freeze import setup, Executable

executables = [Executable("hello.py")]

bdist_msi_options = {
    "summary_data": {
        "author": "Name of the author",
        "comments": "This is a comment",
        "keywords": "These Are Some Keywords",
    },
}

setup(
    name="hello",
    version="0.1",
    description="Sample cx_Freeze script to test MSI summary data stream",
    executables=executables,
    options={
        "build_exe": {"excludes": ["tkinter"]},
        "bdist_msi": bdist_msi_options,
    },
)



"""
This is the first script that is run when cx_Freeze starts up. It simply
determines the name of the initscript that is to be executed.
"""

import os
import sys
from importlib.machinery import (
    EXTENSION_SUFFIXES,
    ExtensionFileLoader,
    ModuleSpec,
    PathFinder,
)


class ExtensionFinder(PathFinder):
    @classmethod
    def find_spec(cls, fullname, path=None, target=None):
        """
        This finder is only for extension modules found within packages that
        are included in the zip file (instead of as files on disk);
        extension modules cannot be found within zip files but are stored in
        the lib subdirectory; if the extension module is found in a package,
        however, its name has been altered so this finder is needed.
        """
        if path is None:
            return None
        suffixes = EXTENSION_SUFFIXES
        for entry in sys.path:
            if ".zip" in entry:
                continue
            for ext in suffixes:
                location = os.path.join(entry, fullname + ext)
                if os.path.isfile(location):
                    loader = ExtensionFileLoader(fullname, location)
                    return ModuleSpec(fullname, loader, origin=location)
        return None


sys.meta_path.append(ExtensionFinder)


def run():
    name = os.path.basename(sys.executable)
    if sys.platform == "win32":
        name, _ = os.path.splitext(name)
    name = os.path.normcase(name).replace(" ", "_")
    name = name.partition("-")[0]
    name = name.partition(".")[0]
    try:
        module = __import__(name + "__init__")
    except ModuleNotFoundError:
        files = []
        for k in __loader__._files:
            if k.endswith("__init__.pyc"):
                k = k.rpartition("__init__")[0]
                if k.isidentifier():
                    files.append(k)
        if len(files) != 1:
            raise RuntimeError(
                "Apparently, the original executable has been renamed to "
                f"'{name}'. When multiple executables are generated, "
                "renaming is not allowed."
            ) from None
        name = files[0]
        module = __import__(name + "__init__")
    module.run()
